'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getParamsFromRoute(route, options) {
  var rx = /:([^/]+)/gi;
  var match = void 0;
  var params = [];

  while (match = rx.exec(route)) {
    // eslint-disable-line no-cond-assign
    var param = match[1];
    var paramOptions = options && options[param] || {};

    params.push({
      name: match[1],
      defaultValue: paramOptions.defaultValue,
      required: paramOptions.required == null ? paramOptions.defaultValue == null : paramOptions.required
    });
  }
  return params;
}

function matches(routeDef, hashList) {
  var params = getParamsFromRoute(routeDef.route, routeDef.options);

  // matcherFn tests for a single route
  var matcherFn = function matcherFn(hash) {
    var routeTokens = _utils2.default.tokenize(routeDef.route);
    var hashTokens = _utils2.default.tokenize(hash.replace(/^#!/, ''));

    var isMatch = !routeTokens.some(function (token, i) {
      if (token[0] === ':' && !_utils2.default.findParam(token, params).required) {
        return false;
      }

      if (hashTokens.length > i && (hashTokens[i] === token || token[0] === ':')) {
        return false;
      }

      return true;
    });

    // If the hash i longer than the route, it's not a match
    if (hashTokens.length > routeTokens.length) return false;

    return isMatch;
  };

  // Adding this to test a string concatenated list of routes
  return function (hash) {
    return hash.split(',').some(matcherFn);
  }(hashList);
}

function link(routeDef, linkState, includeDefaults) {
  var params = getParamsFromRoute(routeDef.route, routeDef.options);

  function resolveToken(token) {
    if (token[0] === ':') {
      var param = _utils2.default.findParam(token, params);
      var stateValue = linkState[param.name] || param.defaultValue;
      return stateValue;
    }
    return token;
  }

  var routeTokens = _utils2.default.tokenize(routeDef.route).map(resolveToken);

  // Check if routeTokens are all defaults
  if (!params.some(function (p) {
    return p.defaultValue !== linkState[p.name] && linkState[p.name] != null;
  })) {
    if (!includeDefaults) {
      return '';
    } // Return empty hash if all values match defaults
  }

  return '' + routeTokens.join('/');
}

// Find routeDef that best matches a state
function bestMatch(routeDefs, matchState) {
  var hashes = routeDefs.map(function (routeDef) {
    return {
      routeDef: routeDef,
      hash: link(routeDef, matchState, true)
    };
  }).filter(function (h) {
    return h.hash;
  });

  return hashes.length ? hashes[0] : null;
}

function state(routeDef, hash) {
  var params = getParamsFromRoute(routeDef.route, routeDef.options);

  // Find the first matching hash if there's a list
  var matchedHash = hash.split(',').find(function (h) {
    return matches(routeDef, h);
  }) || '';

  var routeTokens = _utils2.default.tokenize(routeDef.route);
  var hashTokens = _utils2.default.tokenize(matchedHash.replace(/^#!/, '')).filter(function (token, i) {
    return routeTokens.length > i && routeTokens[i][0] === ':';
  });

  var match = routeTokens.filter(function (token) {
    return token[0] === ':';
  }).map(function (token, i) {
    var result = hashTokens.length > i ? hashTokens[i] : _utils2.default.findParam(token, params).defaultValue;
    return result;
  });

  return _utils2.default.zipObject(params.map(function (p) {
    return p.name;
  }), match);
}

function parseRoute(key, route, options) {
  return {
    key: key,
    route: route,
    options: options
  };
}

exports.default = {
  parseRoute: parseRoute,
  matches: matches,
  state: state,
  link: link,
  getParamsFromRoute: getParamsFromRoute,
  bestMatch: bestMatch
};