'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _reflux = require('reflux');

var _reflux2 = _interopRequireDefault(_reflux);

var _routeParser = require('./route-parser');

var _routeParser2 = _interopRequireDefault(_routeParser);

var _routingActions = require('./routing-actions');

var _routingActions2 = _interopRequireDefault(_routingActions);

var _routingStore = require('./routing-store');

var _routingStore2 = _interopRequireDefault(_routingStore);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var store = _reflux2.default.initStore(_routingStore2.default);

function define(key, route, options) {
  _routingActions2.default.routeDefined({ key: key, route: route, options: options });
}

function link(key, state) {
  var routes = store.state.routes[key];

  if (routes == null) {
    console.warn('Could not create link. Route ' + key + ' not found'); // eslint-disable-line no-console
    return '';
  }

  var allLinks = Object.keys(store.state.routes).filter(function (k) {
    return store.state[k];
  }).map(function (k) {
    if (k === key) {
      var route = _routeParser2.default.bestMatch(routes, state);
      return _routeParser2.default.link(route.routeDef, state, true);
    }

    // Add existing state to the hash link
    var bestMatched = _routeParser2.default.bestMatch(store.state.routes[k], store.state[k]);

    return _routeParser2.default.link(bestMatched.routeDef, store.state[k]);
  });

  return '#!' + allLinks.filter(function (l) {
    return l;
  }).join(',');
}

function navigate(key, state) {
  window.location.hash = link(key, state);
}

function clearState() {
  Object.keys(store.state.routes).forEach(function (key) {
    _routingActions2.default.routeUpdated(key, null);
  });
}

exports.default = {
  define: define,
  link: link,
  clearState: clearState,
  navigate: navigate
};