'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _reflux = require('reflux');

var _reflux2 = _interopRequireDefault(_reflux);

var _routingActions = require('./routing-actions');

var _routingActions2 = _interopRequireDefault(_routingActions);

var _routeParser = require('./route-parser');

var _routeParser2 = _interopRequireDefault(_routeParser);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// Array.prototype.flatMap = Array.prototype.flatMap ||

function flatMap(arr, lambda) {
  return Array.prototype.concat.apply([], arr.map(lambda));
}

function routeConflicts(newRoute) {
  return function (route) {
    var x = newRoute.route.replace(/:[^/]*/gi, '___');
    var y = route.route.replace(/:[^/]*/gi, '___');

    return x === y;
  };
}

var Store = function (_Reflux$Store) {
  _inherits(Store, _Reflux$Store);

  function Store() {
    _classCallCheck(this, Store);

    var _this = _possibleConstructorReturn(this, (Store.__proto__ || Object.getPrototypeOf(Store)).call(this));

    _this.listenables = [_routingActions2.default];
    _this.state = {
      hash: '',
      routes: {}
    };

    _this.setStateFromHash(location.hash);
    return _this;
  }

  _createClass(Store, [{
    key: 'onHashUpdated',
    value: function onHashUpdated(newHash) {
      this.setStateFromHash(newHash);
    }
  }, {
    key: 'onRouteUpdated',
    value: function onRouteUpdated(key, state) {
      var _this2 = this;

      if ((typeof state === 'undefined' ? 'undefined' : _typeof(state)) !== 'object') {
        console.warn('Route update must provide a state and i must be an object (' + key + ')'); // eslint-disable-line no-console
        return;
      }

      this.setState(_defineProperty({}, key, state));

      // To support multiple routes, we should trigger a state change for all route states as some
      // components won't be triggered otherwise
      Object.keys(this.state.routes).forEach(function (route) {
        _this2.setState(_defineProperty({}, route, _this2.state[route]));
      });
    }
  }, {
    key: 'onRouteDefined',
    value: function onRouteDefined(route) {
      var allRoutes = this.state.routes || {};

      allRoutes[route.key] = allRoutes[route.key] || [];

      var conflictingRoute = allRoutes[route.key].find(routeConflicts(route));

      if (!conflictingRoute) {
        allRoutes[route.key].push(route);
      } else {
        allRoutes[route.key][allRoutes[route.key].indexOf(conflictingRoute)] = route;
        console.warn('WARNING: New route \'' + route.route + '\' is conflicting with existing route \'' + conflictingRoute.route + '\'. Existing route will be overwritten '); // eslint-disable-line no-console
      }

      this.setState({
        routes: allRoutes
      });

      var newRouteState = _routeParser2.default.state(route, this.state.hash);
      if (!this.stateEquals(route.key, newRouteState)) {
        _routingActions2.default.routeUpdated(route.key, newRouteState);
      }
    }
  }, {
    key: 'setStateFromHash',
    value: function setStateFromHash(newHash) {
      var _this3 = this;

      var matchedRoutes = this.matched(newHash).filter(function (r) {
        return r;
      });

      this.setState({ hash: newHash });

      matchedRoutes.forEach(function (route) {
        var newState = _routeParser2.default.state(route, newHash);
        if (!_this3.stateEquals(route.key, newState)) {
          _routingActions2.default.routeUpdated(route.key, newState);
        }
      });
    }
  }, {
    key: 'stateEquals',
    value: function stateEquals(key, newState) {
      return JSON.stringify(this.state[key]) === JSON.stringify(newState);
    }
  }, {
    key: 'matched',
    value: function matched(url) {
      var _this4 = this;

      var matchedRoutes = flatMap(Object.keys(this.state.routes), function (routeKey) {
        return _this4.state.routes[routeKey];
      }).filter(function (route) {
        return _routeParser2.default.matches(route, url);
      });
      return matchedRoutes;
    }
  }]);

  return Store;
}(_reflux2.default.Store);

exports.default = Store;